﻿using System.Collections.ObjectModel;
using System.Collections.Specialized;

using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;

namespace TestProject.ViewModels
{
    /// <summary>
    /// 메인 윈도우 뷰 모델
    /// </summary>
    public class MainWindowViewModel : BindableBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 영역 관리자 인터페이스
        /// </summary>
        private readonly IRegionManager regionManager;

        /// <summary>
        /// 제목
        /// </summary>
        private string title = "IRegionMemberLifetime 인터페이스 : KeepAlive 속성을 사용해 메모리에서 자동으로 뷰 제거하기";

        /// <summary>
        /// 뷰 컬렉션
        /// </summary>
        private ObservableCollection<object> viewCollection = new ObservableCollection<object>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 제목 - Title

        /// <summary>
        /// 제목
        /// </summary>
        public string Title
        {
            get
            {
                return this.title;
            }
            set
            {
                SetProperty(ref this.title, value);
            }
        }

        #endregion
        #region 뷰 컬렉션 - ViewCollection

        /// <summary>
        /// 뷰 컬렉션
        /// </summary>
        public ObservableCollection<object> ViewCollection
        {
            get
            {
                return this.viewCollection;
            }
            set
            {
                SetProperty(ref this.viewCollection, value);
            }
        }

        #endregion
        #region 탐색 명령 - NavigateCommand

        /// <summary>
        /// 탐색 명령
        /// </summary>
        public DelegateCommand<string> NavigateCommand { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindowViewModel(regionManager)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="regionManager">영역 관리자 인터페이스</param>
        public MainWindowViewModel(IRegionManager regionManager)
        {
            this.regionManager = regionManager;

            this.regionManager.Regions.CollectionChanged += regionCollection_CollectionChanged;

            NavigateCommand = new DelegateCommand<string>(Navigate);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 뷰 컬렉션 컬렉션 변경시 처리하기 - viewCollection_CollectionChanged(sender, e)

        /// <summary>
        /// 뷰 컬렉션 컬렉션 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void viewCollection_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if(e.Action == NotifyCollectionChangedAction.Add)
            {
                ViewCollection.Add(e.NewItems[0].GetType().Name);
            }
            else if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                ViewCollection.Remove(e.OldItems[0].GetType().Name);
            }
        }

        #endregion
        #region 영역 컬렉션 컬렉션 변경시 처리하기 - regionCollection_CollectionChanged(sender, e)

        /// <summary>
        /// 영역 컬렉션 컬렉션 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void regionCollection_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if(e.Action == NotifyCollectionChangedAction.Add)
            {
                IRegion region = e.NewItems[0] as IRegion;

                region.Views.CollectionChanged += viewCollection_CollectionChanged;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 탐색하기 - Navigate(navigatePath)

        /// <summary>
        /// 탐색하기
        /// </summary>
        /// <param name="navigatePath">탐색 경로</param>
        private void Navigate(string navigatePath)
        {
            if(navigatePath != null)
            {
                regionManager.RequestNavigate("ContentRegion", navigatePath);
            }
        }

        #endregion
    }
}